/*
 * Class name: WriteToFc2
 * Date:       Mar/03/98
 * Author:     Yung-Pin Cheng
 * Description: This class takes an directed graph to output a FC2 file
 * Methods:    
 *
 */


import java.io.*;

public class WriteToFc2 {
  PrintWriter fp = null ;
  static final int MAXNAMES = 500 ;
  String[] hashname = new String[MAXNAMES];
  int hashno = 0 ;
  int start = -1 ;

  public WriteToFc2 (String filename) {
    try {
      FileOutputStream out = new FileOutputStream(filename) ;
      fp = new PrintWriter(out);
    } catch (IOException e) {
      System.out.println("ERROR: cannot open " + filename );
    }
  }
  public void writeFc2Files (DiGraph gr) {
    int i;
    Node p = gr.node ;
    Edge e ;
    int eno, ctr = 0 ;
    fp.println("% Fc2 file generated from dot2fc2 by Yung-Pin Cheng");
    fp.println("version \"1.1\"");
    fp.println("nets 1");
    fp.println("hook\"main\" > 0");
    fp.println("struct\""+gr.name+"\"");
    fp.println("net 0");
    
    // we build a table to process those edges having the same name except
    // parameters
    e = gr.edge ;
    while (e != null ) {
      if (e.deleted) {
	e = e.next ; 
	continue ;
      }
      if (e.type == Edge.eta || e.name.compareTo("delta")==0) {
	e = e.next ; 
	continue ;
      }
      for (i=0;i< hashno;i++) {
	if (e.name.compareTo(hashname[i]) == 0 )
	  break ;
      }
      if (i >= hashno) { // a new task.entry
	hashname[hashno] = new String(e.name);
	hashno++ ;
      }
      e = e.next ;
    }
  
    // print call edge 
    fp.println("B"+hashno);
    for (i=0;i< hashno; i++) {
      TaskString name = new TaskString(hashname[i]);
      String task = name.getTask();
      String entry = name.getEntry();
      String para = name.getPara();
      fp.println(":"+i+ " \"" + becomeXXYY(hashname[i],task,entry,para)+ "\"");
    }
    fp.println() ;

    // print initial node and node number 
    p = gr.node ;
    ctr = 0 ;
    while (p != null ) {
      p.offset = ctr ;
      if (p == gr.start) 
	start = ctr ;
      p = p.next ;
      ctr ++ ;

    }
    fp.println("l \"initial\">"+ start + " h \"automaton\"");
    fp.println("V"+ctr);


    // print outgoing edges of each node 
    p = gr.node ;
    ctr = 0 ;
    while (p != null ) {
      EdgeWrap oe  = p.out ;
      eno = 0 ;
      // print out vertiex index and edge number
      while (oe != null) {
	oe = oe.next ;
	eno++;
      }
      fp.println("v"+ctr+ " E"+eno);
      eno = 0 ; oe = p.out ;
      while (oe != null ) {
	if (oe.edge.type == Edge.eta ) 
	  fp.println("e"+eno+ " b "+ "tau r "+  oe.edge.tl.offset);
	else if (oe.edge.name.compareTo("delta") == 0) 
	  fp.println("e"+eno+ " b "+ "delta r "+  oe.edge.tl.offset);	  
	else {
	  TaskString name = new TaskString(oe.edge.name);
	  String task = name.getTask();
	  String entry = name.getEntry();
	  String para = name.getPara();	  
	  fp.println("e"+eno+ " b "+ "\""+
		     becomeXXYY(oe.edge.name, task, entry, para)
		     +"\" r "+ 
		     oe.edge.tl.offset);
	}
	oe = oe.next ;
	eno++ ;
      }
      p = p.next ;
      ctr ++ ;
    }
    fp.close();
  }	
  private String becomeXXYY(String orig, String task, 
			  String entry, String para) {
    
    if (task == null && entry == null && para == null)
      return orig ;
    else {
      if (para != null )
	return new String(task +"XX"+entry+"YY"+
			  insertZZ(para));
      else
	return new String(task + "XX"+ entry);
    }    
  }
  public String insertZZ(String str) {
    int start = 0 ;
    String rtstr = new String("");
    for (int i=0;i< str.length(); i++) {
      if (str.charAt(i) == ',' ) {
	rtstr = new String(rtstr + str.substring(start,i) + "ZZ");
	start = i + 1 ;
      }
    }
    rtstr = new String(rtstr + str.substring(start, str.length()) );
    return rtstr ;
  }

}







